@@ -17,8 +17,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import dbus.service
-import logging
 
 XDG_CONFIG_HOME = os.environ.get('XDG_CONFIG_HOME', os.path.expanduser('~/.config'))
 # Runtime dir falls back to cache dir, as a fallback is suggested by the spec
@@ -32,15 +30,10 @@ DATA_DIR = os.path.join(XDG_DATA_HOME, "autokey")
 AUTOSTART_DIR = os.path.join(XDG_CONFIG_HOME, "autostart")
 
 LOCK_FILE = os.path.join(RUN_DIR, "autokey.pid")
-LOG_FILE = os.path.join(DATA_DIR, "autokey.log")
-
-MAX_LOG_SIZE = 5 * 1024 * 1024  # 5 megabytes
-MAX_LOG_COUNT = 3
-LOG_FORMAT = "%(asctime)s %(levelname)s - %(name)s - %(message)s"
 
 APP_NAME = "autokey"
 CATALOG = ""
-VERSION = "0.95.10"
+VERSION = "0.96.0"
 HOMEPAGE = "https://github.com/autokey/autokey"
 AUTHOR = 'Chris Dekter'
 AUTHOR_EMAIL = 'cdekter@gmail.com'
@@ -59,28 +52,3 @@ ICON_FILE_NOTIFICATION_DARK = "autokey-status-dark"
 ICON_FILE_NOTIFICATION_ERROR = "autokey-status-error"
 
 USING_QT = False
-
-
-class AppService(dbus.service.Object):
-
-    def __init__(self, app):
-        busName = dbus.service.BusName('org.autokey.Service', bus=dbus.SessionBus())
-        dbus.service.Object.__init__(self, busName, "/AppService")
-        self.app = app
-        logging.debug("Created DBus service")
-
-    @dbus.service.method(dbus_interface='org.autokey.Service', in_signature='', out_signature='')
-    def show_configure(self):
-        self.app.show_configure()
-
-    @dbus.service.method(dbus_interface='org.autokey.Service', in_signature='s', out_signature='')
-    def run_script(self, name):
-        self.app.service.run_script(name)
-
-    @dbus.service.method(dbus_interface='org.autokey.Service', in_signature='s', out_signature='')
-    def run_phrase(self, name):
-        self.app.service.run_phrase(name)
-
-    @dbus.service.method(dbus_interface='org.autokey.Service', in_signature='s', out_signature='')
-    def run_folder(self, name):
-        self.app.service.run_folder(name)
