@@ -15,18 +15,33 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>..
 
+import datetime
 import threading
 
 import gi
+
+
 gi.require_version('Gtk', '3.0')
 gi.require_version('Notify', '0.7')
-gi.require_version('AppIndicator3', '0.1')
+# AppIndicator is apparently deprecated, and on Debian the namespace is now
+# `AyatanaAppIndicator3`, which is a maintained verison.
+try:
+    gi.require_version('AyatanaAppIndicator3', '0.1')
+except ValueError:
+    gi.require_version('AppIndicator3', '0.1')
 
-from gi.repository import Gtk, Gdk, Notify, AppIndicator3
+from gi.repository import Gtk, Gdk, Notify
+try:
+    from gi.repository import AyatanaAppIndicator3 as AppIndicator
+except ImportError:
+    from gi.repository import AppIndicator3 as AppIndicator
 import gettext
 
 from . import popupmenu
-from .. import configmanager as cm
+
+import autokey.configmanager.configmanager as cm
+import autokey.configmanager.configmanager_constants as cm_constants
+
 from .. import common
 
 
@@ -42,24 +57,32 @@ class IndicatorNotifier:
     
     def __init__(self, autokeyApp):
         Notify.init("AutoKey")
+        # Used to rate-limit error notifications to 1 per second. Without this, two notifications per second cause the
+        # following exception, which in turn completely locks up the GUI:
+        # gi.repository.GLib.GError: g-io-error-quark:
+        # GDBus.Error:org.freedesktop.Notifications.Error.ExcessNotificationGeneration:
+        # Created too many similar notifications in quick succession (36)
+        self.last_notification_timestamp = datetime.datetime.now()
         self.app = autokeyApp
         self.configManager = autokeyApp.service.configManager
 
-        self.indicator = AppIndicator3.Indicator.new("AutoKey", cm.ConfigManager.SETTINGS[cm.NOTIFICATION_ICON],
-                                                AppIndicator3.IndicatorCategory.APPLICATION_STATUS)
+        self.indicator = AppIndicator.Indicator.new(
+            "AutoKey",
+            cm.ConfigManager.SETTINGS[cm_constants.NOTIFICATION_ICON],
+            AppIndicator.IndicatorCategory.APPLICATION_STATUS)
                                                 
         self.indicator.set_attention_icon(common.ICON_FILE_NOTIFICATION_ERROR)
         self.update_visible_status()           
         self.rebuild_menu()
         
     def update_visible_status(self):
-        if cm.ConfigManager.SETTINGS[cm.SHOW_TRAY_ICON]:
-            self.indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
+        if cm.ConfigManager.SETTINGS[cm_constants.SHOW_TRAY_ICON]:
+            self.indicator.set_status(AppIndicator.IndicatorStatus.ACTIVE)
         else:
-            self.indicator.set_status(AppIndicator3.IndicatorStatus.PASSIVE)   
+            self.indicator.set_status(AppIndicator.IndicatorStatus.PASSIVE)   
             
     def hide_icon(self):     
-        self.indicator.set_status(AppIndicator3.IndicatorStatus.PASSIVE)
+        self.indicator.set_status(AppIndicator.IndicatorStatus.PASSIVE)
         
     def set_icon(self,name):
         self.indicator.set_icon(name)
@@ -115,9 +138,12 @@ class IndicatorNotifier:
         self.indicator.set_menu(self.menu)
         
     def notify_error(self, message):
-        self.show_notify(message, Gtk.STOCK_DIALOG_ERROR)
+        now = datetime.datetime.now()
+        if self.last_notification_timestamp + datetime.timedelta(seconds=1) < now:
+            self.show_notify(message, Gtk.STOCK_DIALOG_ERROR)
+            self.last_notification_timestamp = now
         self.errorItem.show()
-        self.indicator.set_status(AppIndicator3.IndicatorStatus.ATTENTION)
+        self.indicator.set_status(AppIndicator.IndicatorStatus.ATTENTION)
         
     def show_notify(self, message, iconName):
         Gdk.threads_enter()
@@ -131,10 +157,11 @@ class IndicatorNotifier:
         
     def on_show_error(self, widget, data=None):
         # Work around the current GUI design: the UI is destroyed when the main window is closed.
-        # This causes the show_script_error method below to fail. So open the main window first to circumvent issues.
-        # TODO: If the GTK GUI gets rewritten to not always destroy it’s UI, remove this comment and workaround.
-        self.on_show_configure(widget, data)
-        self.app.show_script_error(self.app.configWindow.ui)
+        # This causes the show_script_error method below to fail because self.app.configWindow.ui doesn’t exist
+        if self.app.configWindow is not None:
+            self.app.show_script_error(self.app.configWindow.ui)
+        else:
+            self.app.show_script_error(None)
         self.errorItem.hide()
         self.update_visible_status()
             
@@ -148,8 +175,8 @@ class IndicatorNotifier:
         self.app.show_configure()
 
     def on_remove_icon(self, widget, data=None):
-        self.indicator.set_status(AppIndicator3.IndicatorStatus.PASSIVE)
-        cm.ConfigManager.SETTINGS[cm.SHOW_TRAY_ICON] = False
+        self.indicator.set_status(AppIndicator.IndicatorStatus.PASSIVE)
+        cm.ConfigManager.SETTINGS[cm_constants.SHOW_TRAY_ICON] = False
                 
     def on_destroy_and_exit(self, widget, data=None):
         self.app.shutdown()
