@@ -16,13 +16,18 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
-from gi.repository import Gtk
+from gi.repository import Gtk, GtkSource
+
+
+import autokey.configmanager.autostart
+import autokey.configmanager.configmanager as cm
+import autokey.configmanager.configmanager_constants as cm_constants
+from autokey import common
+from autokey.model.key import Key
 
-from autokey import configmanager as cm, common
-from autokey.iomediator.key import Key
 
 from .dialogs import GlobalHotkeyDialog
-from .configwindow0 import get_ui
+from .shared import get_ui
 
 
 ICON_NAME_MAP = {
@@ -55,6 +60,18 @@ class SettingsDialog:
         # Added by Trey Blancher (ectospasm) 2015-09-16
         self.triggerItemByInitial = builder.get_object("triggerItemByInitial")
         self.enableUndoCheckbox = builder.get_object("enableUndoCheckbox")
+
+        self.gtkThemeCombo = Gtk.ComboBoxText.new()
+        hboxgtktheme = builder.get_object("hboxgtktheme")
+        hboxgtktheme.pack_start(self.gtkThemeCombo, False, True, 0)
+        hboxgtktheme.show_all()
+
+        self.themeList = GtkSource.StyleSchemeManager().get_scheme_ids()
+        for item in self.themeList:
+            self.gtkThemeCombo.append_text(item)
+
+        self.gtkThemeCombo.set_sensitive(cm.ConfigManager.SETTINGS[cm_constants.GTK_THEME])
+        self.gtkThemeCombo.set_active(self.themeList.index(cm.ConfigManager.SETTINGS[cm_constants.GTK_THEME]))
         
         self.iconStyleCombo = Gtk.ComboBoxText.new()
         hbox = builder.get_object("hbox4")
@@ -64,21 +81,20 @@ class SettingsDialog:
         for key, value in list(ICON_NAME_MAP.items()):
             self.iconStyleCombo.append_text(key)
             ICON_NAME_LIST.append(value)
-        self.iconStyleCombo.set_sensitive(cm.ConfigManager.SETTINGS[cm.SHOW_TRAY_ICON])
-        self.iconStyleCombo.set_active(ICON_NAME_LIST.index(cm.ConfigManager.SETTINGS[cm.NOTIFICATION_ICON]))
-        
-        self.autoStartCheckbox.set_active(cm.get_autostart().desktop_file_name is not None)
-        self.autosaveCheckbox.set_active(not cm.ConfigManager.SETTINGS[cm.PROMPT_TO_SAVE])
-        self.showTrayCheckbox.set_active(cm.ConfigManager.SETTINGS[cm.SHOW_TRAY_ICON])
+
+        self.iconStyleCombo.set_sensitive(cm.ConfigManager.SETTINGS[cm_constants.SHOW_TRAY_ICON])
+        self.iconStyleCombo.set_active(ICON_NAME_LIST.index(cm.ConfigManager.SETTINGS[cm_constants.NOTIFICATION_ICON]))
+
+        self.autoStartCheckbox.set_active(autokey.configmanager.autostart.get_autostart().desktop_file_name is not None)
+        self.autosaveCheckbox.set_active(not cm.ConfigManager.SETTINGS[cm_constants.PROMPT_TO_SAVE])
+        self.showTrayCheckbox.set_active(cm.ConfigManager.SETTINGS[cm_constants.SHOW_TRAY_ICON])
         self.disableCapslockCheckbox.set_active(cm.ConfigManager.is_modifier_disabled(Key.CAPSLOCK))
-        #self.allowKbNavCheckbox.set_active(cm.ConfigManager.SETTINGS[MENU_TAKES_FOCUS])
+
+        # self.allowKbNavCheckbox.set_active(cm.ConfigManager.SETTINGS[MENU_TAKES_FOCUS])
         # Added by Trey Blancher (ectospasm) 2015-09-16
-        self.triggerItemByInitial.set_active(cm.ConfigManager.SETTINGS[cm.TRIGGER_BY_INITIAL])
-        self.sortByUsageCheckbox.set_active(cm.ConfigManager.SETTINGS[cm.SORT_BY_USAGE_COUNT])
-        self.enableUndoCheckbox.set_active(cm.ConfigManager.SETTINGS[cm.UNDO_USING_BACKSPACE])
-        
-
-
+        self.triggerItemByInitial.set_active(cm.ConfigManager.SETTINGS[cm_constants.TRIGGER_BY_INITIAL])
+        self.sortByUsageCheckbox.set_active(cm.ConfigManager.SETTINGS[cm_constants.SORT_BY_USAGE_COUNT])
+        self.enableUndoCheckbox.set_active(cm.ConfigManager.SETTINGS[cm_constants.UNDO_USING_BACKSPACE])
 
         # Hotkeys
         self.showConfigDlg = GlobalHotkeyDialog(parent, configManager, self.on_config_response)
@@ -89,9 +105,9 @@ class SettingsDialog:
         self.clearMonitorButton = builder.get_object("clearMonitorButton")    
         
         self.useConfigHotkey = self.__loadHotkey(configManager.configHotkey, self.configKeyLabel, 
-                                                    self.showConfigDlg, self.clearConfigButton)
+                                                 self.showConfigDlg, self.clearConfigButton)
         self.useServiceHotkey = self.__loadHotkey(configManager.toggleServiceHotkey, self.monitorKeyLabel, 
-                                                    self.toggleMonitorDlg, self.clearMonitorButton)
+                                                  self.toggleMonitorDlg, self.clearMonitorButton)
                                                     
         # Script Engine Settings
         self.userModuleChooserButton = builder.get_object("userModuleChooserButton")
@@ -102,18 +118,22 @@ class SettingsDialog:
 
     def on_save(self, widget, data=None):
         if self.autoStartCheckbox.get_active():
-            cm.set_autostart_entry(cm.AutostartSettings("autokey-gtk.desktop", False))
+            autokey.configmanager.autostart.set_autostart_entry(
+                autokey.configmanager.autostart.AutostartSettings("autokey-gtk.desktop", False))
         else:
-            cm.delete_autostart_entry()
-    
-        cm.ConfigManager.SETTINGS[cm.PROMPT_TO_SAVE] = not self.autosaveCheckbox.get_active()
-        cm.ConfigManager.SETTINGS[cm.SHOW_TRAY_ICON] = self.showTrayCheckbox.get_active()
+            autokey.configmanager.autostart.delete_autostart_entry()
+
+
+        #promptToSaveCheckbox no longer exists? This prevented saving in the Preferences window from what I can tell
+        #cm.ConfigManager.SETTINGS[cm_constants.PROMPT_TO_SAVE] = not self.promptToSaveCheckbox.get_active()
+        cm.ConfigManager.SETTINGS[cm_constants.SHOW_TRAY_ICON] = self.showTrayCheckbox.get_active()
+        cm.ConfigManager.SETTINGS[cm_constants.GTK_THEME] = self.gtkThemeCombo.get_active_text()
         #cm.ConfigManager.SETTINGS[MENU_TAKES_FOCUS] = self.allowKbNavCheckbox.get_active()
-        cm.ConfigManager.SETTINGS[cm.SORT_BY_USAGE_COUNT] = self.sortByUsageCheckbox.get_active()
+        cm.ConfigManager.SETTINGS[cm_constants.SORT_BY_USAGE_COUNT] = self.sortByUsageCheckbox.get_active()
         # Added by Trey Blancher (ectospasm) 2015-09-16
-        cm.ConfigManager.SETTINGS[cm.TRIGGER_BY_INITIAL] = self.triggerItemByInitial.get_active()
-        cm.ConfigManager.SETTINGS[cm.UNDO_USING_BACKSPACE] = self.enableUndoCheckbox.get_active()
-        cm.ConfigManager.SETTINGS[cm.NOTIFICATION_ICON] = ICON_NAME_MAP[self.iconStyleCombo.get_active_text()]
+        cm.ConfigManager.SETTINGS[cm_constants.TRIGGER_BY_INITIAL] = self.triggerItemByInitial.get_active()
+        cm.ConfigManager.SETTINGS[cm_constants.UNDO_USING_BACKSPACE] = self.enableUndoCheckbox.get_active()
+        cm.ConfigManager.SETTINGS[cm_constants.NOTIFICATION_ICON] = ICON_NAME_MAP[self.iconStyleCombo.get_active_text()]
         self._save_disable_capslock_setting()
         self.configManager.userCodeDir = self.userModuleChooserButton.get_current_folder()
         sys.path.append(self.configManager.userCodeDir)
