@@ -14,35 +14,38 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
-import logging
-import threading
-import time
 import webbrowser
 
+from PyQt5.QtCore import pyqtSignal, QTimer
 from PyQt5.QtGui import QIcon, QKeySequence, QCloseEvent
 from PyQt5.QtWidgets import QApplication, QAction, QMenu
 
 
 import autokey.common
+import autokey.model.folder
+import autokey.model.phrase
+import autokey.model.script
 import autokey.qtui.common
-from autokey import configmanager as cm
-from autokey import model
+import autokey.configmanager.configmanager as cm
+import autokey.configmanager.configmanager_constants as cm_constants
+
 from .settings import SettingsDialog
 from . import dialogs
 
+logger = __import__("autokey.logger").logger.get_logger(__name__)
 PROBLEM_MSG_PRIMARY = "Some problems were found"
 PROBLEM_MSG_SECONDARY = "%1\n\nYour changes have not been saved."
 
-_logger = autokey.qtui.common.logger.getChild("configwindow")  # type: logging.Logger
-
 
 class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwindow")):
 
+    script_errors_available = pyqtSignal(bool, name="script_errors_available")
+
     def __init__(self, app: QApplication):
         super().__init__()
         self.setupUi(self)
         self.about_dialog = dialogs.AboutAutokeyDialog(self)
+        self.show_script_errors_dialog = self._create_show_recent_script_errors_dialog()
         self.app = app
         self.action_create = self._create_action_create()
         self.toolbar.insertAction(self.action_save, self.action_create)  # Insert before action_save, i.e. at index 0
@@ -74,6 +77,12 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
         action_create.setMenu(create_menu)
         return action_create
 
+    def _create_show_recent_script_errors_dialog(self) -> dialogs.ShowRecentScriptErrorsDialog:
+        show_script_errors_dialog = dialogs.ShowRecentScriptErrorsDialog(self)
+        # Forward the signal from the dialog instance to the own signal
+        show_script_errors_dialog.script_errors_available.connect(self.script_errors_available)
+        return show_script_errors_dialog
+
     def _connect_all_file_menu_signals(self):
         # Show the action_create popup menu regardless where the user places the click.
         # The Action is displayed as "[<icon>]v". Clicking on the downwards arrow opens the popup menu as
@@ -102,8 +111,12 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
         self.action_rename_item.triggered.connect(self.central_widget.on_rename)
 
     def _connect_all_tools_menu_signals(self):
-        self.action_show_last_script_error.triggered.connect(self.app.notifier.reset_tray_icon)
-        self.action_show_last_script_error.triggered.connect(self.app.show_script_error)
+        self.action_show_last_script_errors.triggered.connect(self.show_script_errors_dialog.update_and_show)
+        # Only enable action_show_last_script_errors if script errors are recorded.
+        # Automatically disable the action if no errors are viewable to prevent the user from seeing a dialogue window
+        # in some undefined state.
+        self.script_errors_available.connect(self.action_show_last_script_errors.setEnabled)
+
         self.action_record_script.triggered.connect(self.on_record)
         self.action_run_script.triggered.connect(self.on_run_script)
         # Add all defined macros to the »Insert Macros« menu
@@ -192,10 +205,10 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
         
     def update_actions(self, items, changed):
         if len(items) > 0:
-            can_create = isinstance(items[0], model.Folder) and len(items) == 1
+            can_create = isinstance(items[0], autokey.model.folder.Folder) and len(items) == 1
             can_copy = True
             for item in items:
-                if isinstance(item, model.Folder):
+                if isinstance(item, autokey.model.folder.Folder):
                     can_copy = False
                     break
             
@@ -207,9 +220,9 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
             self.action_copy_item.setEnabled(can_copy)
             self.action_clone_item.setEnabled(can_copy)
             self.action_paste_item.setEnabled(can_create and len(self.central_widget.cutCopiedItems) > 0)
-            self.action_record_script.setEnabled(isinstance(items[0], model.Script) and len(items) == 1)
-            self.action_run_script.setEnabled(isinstance(items[0], model.Script) and len(items) == 1)
-            self.menu_insert_macros.setEnabled(isinstance(items[0], model.Phrase) and len(items) == 1)
+            self.action_record_script.setEnabled(isinstance(items[0], autokey.model.script.Script) and len(items) == 1)
+            self.action_run_script.setEnabled(isinstance(items[0], autokey.model.script.Script) and len(items) == 1)
+            self.menu_insert_macros.setEnabled(isinstance(items[0], autokey.model.phrase.Phrase) and len(items) == 1)
 
             if changed:
                 self.action_save.setEnabled(False)
@@ -223,7 +236,7 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
         self.action_redo.setEnabled(state)
 
     def save_completed(self, persist_global):
-        _logger.debug("Saving completed. persist_global: {}".format(persist_global))
+        logger.debug("Saving completed. persist_global: {}".format(persist_global))
         self.action_save.setEnabled(False)
         self.app.config_altered(persist_global)
         
@@ -235,8 +248,8 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
     # ---- Signal handlers ----
     
     def queryClose(self):
-        cm.ConfigManager.SETTINGS[cm.HPANE_POSITION] = self.central_widget.splitter.sizes()[0] + 4
-        cm.ConfigManager.SETTINGS[cm.COLUMN_WIDTHS] = [
+        cm.ConfigManager.SETTINGS[cm_constants.HPANE_POSITION] = self.central_widget.splitter.sizes()[0] + 4
+        cm.ConfigManager.SETTINGS[cm_constants.COLUMN_WIDTHS] = [
             self.central_widget.treeWidget.columnWidth(column_index) for column_index in range(3)
         ]
         
@@ -245,7 +258,6 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
                 return False
 
         self.hide()
-        # logging.getLogger().removeHandler(self.central_widget.logHandler)
         return True
     
     # File Menu
@@ -280,14 +292,14 @@ class ConfigWindow(*autokey.qtui.common.inherits_from_ui_file_with_name("mainwin
             self.action_record_script.setChecked(False)
 
     def on_run_script(self):
-        t = threading.Thread(target=self._run_script)
-        t.start()
-
-    def _run_script(self):
         script = self.central_widget.get_selected_item()[0]
-        time.sleep(2)  # Fix the GUI tooltip for action_run_script when changing this!
-        self.app.service.scriptRunner.execute(script)
-    
+        QTimer.singleShot(
+            2000,  # Fix the GUI tooltip for action_run_script when changing this!
+            (lambda: self.app.service.scriptRunner.execute_script(
+                script
+            ))
+        )
+
     # Settings Menu
             
     def on_advanced_settings(self):
